/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlayout.h>
#include <qfile.h>
#include <qdragobject.h>
#include <qsplitter.h>
#include <qtextedit.h>
#include <qprogressdialog.h>
#include <qbuffer.h>
#include <qapplication.h>

#include <klocale.h>
#include <kurl.h>
#include <ktempfile.h>
#include <kglobalsettings.h>
#include <kio/netaccess.h>
#include <kapplication.h>

#include <searchbar.h>
#include <documentlistview.h>
#include <documentlistviewitem.h>
#include <documentsourceview.h>
#include <stringwidget.h>
#include <commentwidget.h>
#include <entrywidget.h>
#include <bibtexfileimporterbibtex.h>
#include <bibtexfileexporterbibtex.h>
#include <bibtexfileexporterxml.h>
#include <bibtexfileexporterpdf.h>
#include <bibtexfileexporterps.h>
#include <bibtexfileexporterxslt.h>
#include <bibtexfileimporterexternal.h>
#include <bibtexfileexporterexternal.h>
#include <bibtexxsltransform.h>
#include <bibtexentry.h>
#include <bibtexcomment.h>
#include <bibtexstring.h>
#include <settings.h>

#include "documentwidget.h"

namespace KBibTeX
{

    DocumentWidget::DocumentWidget( bool isReadOnly, QWidget *parent, const char *name )
            : QTabWidget( parent, name ), m_isReadOnly( isReadOnly ), m_progressDialog( NULL ), m_newElementCounter( 1 )
    {
        m_bibtexfile = new BibTeX::BibTeXFile();

        setupGUI();
    }

    DocumentWidget::~DocumentWidget()
    {
        delete m_bibtexfile;
    }

    void DocumentWidget::setupGUI()
    {
        setAcceptDrops( TRUE );
        setFocusPolicy( QWidget::ClickFocus );

        // List view tab widget ===============================

        m_verticalContainer = new QWidget( this );
        QBoxLayout *layout = new QVBoxLayout( m_verticalContainer, 0, 0 );
        addTab( m_verticalContainer, i18n( "List view" ) );

        m_searchBar = new KBibTeX::SearchBar( m_verticalContainer );
        layout->addWidget( m_searchBar );

        QSplitter *split = new QSplitter( Qt::Vertical, m_verticalContainer );
        layout->addWidget( split );

        m_listViewElements = new DocumentListView( this, m_isReadOnly, split );

        m_labelPreview = new QLabel( split );
        m_labelPreview->setAlignment( Qt::AlignTop && Qt::AlignLeft );

        // Source view tab widget =============================

        m_sourceView = new DocumentSourceView( this, m_isReadOnly, this , "source_view" );
        addTab( m_sourceView, i18n( "Source view" ) );
        m_sourceView->setFont( KGlobalSettings::fixedFont() );

        // Setup actions
        connect( m_searchBar, SIGNAL( doSearch( const QString& ) ), m_listViewElements, SLOT( filterText( const QString& ) ) );
        connect ( m_listViewElements, SIGNAL( selectionChanged() ), this, SLOT( slotSelectionChanged() ) );
        connect( m_listViewElements, SIGNAL( currentChanged( QListViewItem* ) ), this, SLOT( slotPreviewElement( QListViewItem* ) ) );
        connect( m_listViewElements, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( slotPreviewElement( QListViewItem* ) ) );
        connect( this, SIGNAL( currentChanged ( QWidget * ) ), this, SLOT( slotTabChanged( QWidget* ) ) );
        connect( m_sourceView, SIGNAL( modified() ), this, SLOT( slotModified() ) );
        connect( m_listViewElements, SIGNAL( modified() ), this, SLOT( slotModified() ) );
    }


    bool DocumentWidget::open( const QString &fileName, bool insertOnly )
    {
        bool result = FALSE;

        Settings::FileFormat format = Settings::formatUndefined;
        if ( fileName.endsWith( ".bib" ) )
            format = Settings::formatBibTeX;
        else
            format = Settings::formatUndefined;

        if ( format != Settings::formatUndefined )
        {
            BibTeX::BibTeXFileImporter * importer = NULL;
            switch ( format )
            {
            case Settings::formatBibTeX:
                importer = new BibTeX::BibTeXFileImporterBibTeX();
                break;
            default:
                {
                    // ignore, as other formats cannot be imported
                }
            }

            if ( importer != NULL )
            {
                QFile file( fileName );
                if ( file.open( IO_ReadOnly ) )
                {
                    result = open( &file, insertOnly, QString( i18n( "<qt>Loading file <b>%1</b></qt>" ) ).arg( fileName ), importer );
                    file.close();
                }
                delete importer;
            }
        }

        return result;
    }

    bool DocumentWidget::open( QIODevice *iodevice, bool insertOnly, const QString &label, BibTeX::BibTeXFileImporter *importer )
    {
        bool result = FALSE;
        setEnabled( FALSE );

        bool usingDefaultImporter = importer == NULL;
        if ( usingDefaultImporter )
            importer = new BibTeX::BibTeXFileImporterBibTeX();

        startProgress( label, importer );
        BibTeX::BibTeXFile *newFile = importer->load( iodevice );
        endProgress( importer );

        if ( usingDefaultImporter )
            delete importer;

        if ( newFile != NULL )
        {
            if ( insertOnly )
            {
                m_bibtexfile->append( newFile, NULL );
                delete newFile;
            }
            else
            {
                delete m_bibtexfile;
                m_bibtexfile = newFile;
            }

            m_listViewElements->setBibTeXFile( m_bibtexfile );
            m_sourceView->setBibTeXFile( m_bibtexfile );
            result = TRUE;
        }
        else
        {
            qDebug( "Could not load bibliography file from io device" );
        }

        setEnabled( TRUE );
        return result;
    }

    bool DocumentWidget::save( const QString &fileName, QStringList *errorLog )
    {
        bool result = FALSE;

        Settings::FileFormat format = Settings::formatUndefined;
        if ( fileName.endsWith( ".pdf", FALSE ) )
            format = Settings::formatPDF;
        else if ( fileName.endsWith( ".bib", FALSE ) )
            format = Settings::formatBibTeX;
        else if ( fileName.endsWith( ".ps", FALSE ) )
            format = Settings::formatPS;
        else if ( fileName.endsWith( ".xml", FALSE ) )
            format = Settings::formatXML;
        else if ( fileName.endsWith( ".html", FALSE ) || fileName.endsWith( ".xhtml", FALSE ) || fileName.endsWith( ".htm", FALSE ) )
            format = Settings::formatHTML;
        if ( format == Settings::formatUndefined )
            return FALSE;

        QFile file( fileName );
        if ( file.open( IO_WriteOnly ) )
        {
            result = save( &file, format, QString( i18n( "<qt>Writing file <b>%1</b></qt>" ) ).arg( fileName ), errorLog );
            file.close();
        }

        return result;
    }

    bool DocumentWidget::save( QIODevice *iodevice, Settings::FileFormat format, const QString &label, QStringList *errorLog )
    {
        bool result = FALSE;
        setEnabled( FALSE );

        updateFromGUI();

        BibTeX::BibTeXXSLTransform * transform = NULL;

        BibTeX::BibTeXFileExporter * exporter = NULL;
        switch ( format )
        {
        case Settings::formatBibTeX:
            {
                BibTeX::BibTeXFileExporterBibTeX * bibtexExporter = new BibTeX::BibTeXFileExporterBibTeX();
                bibtexExporter->setStringDelimiter( kbibtexsettings.bibtexStringOpenDelimiter, kbibtexsettings.bibtexStringCloseDelimiter );
                bibtexExporter->setEncoding( kbibtexsettings.encoding );
                exporter = bibtexExporter;
            }
            break;
        case Settings::formatXML:
            exporter = new BibTeX::BibTeXFileExporterXML();
            break;
        case Settings::formatHTML:
            switch ( kbibtexsettings.usedExporterHTML )
            {
            case Settings::exporterNone:
            case Settings::exporterXSLT:
                {
                    transform = new BibTeX::BibTeXXSLTransform( kbibtexsettings.xsltStylesheetHTML );
                    if ( transform != NULL )
                        exporter = new BibTeX::BibTeXFileExporterXSLT( transform );
                    else
                    {
                        qDebug( "XSLT for HTML export is not available" );
                    }
                }
                break;
            default:
                exporter = new BibTeX::BibTeXFileExporterExternal( kbibtexsettings.usedExporterHTML, Settings::formatHTML );
            }
            break;
        case Settings::formatPDF:
            {
                BibTeX::BibTeXFileExporterPDF *pdfExporter = new BibTeX::BibTeXFileExporterPDF();
                pdfExporter->setLaTeXLanguage( KBibTeX::kbibtexsettings.exportlanguage );
                pdfExporter->setLaTeXBibliographyStyle( KBibTeX::kbibtexsettings.exportbibliographystyle );
                exporter = pdfExporter;
            }
            break;
        case Settings::formatPS:
            {
                BibTeX::BibTeXFileExporterPS *psExporter = new BibTeX::BibTeXFileExporterPS();
                psExporter->setLaTeXLanguage( KBibTeX::kbibtexsettings.exportlanguage );
                psExporter->setLaTeXBibliographyStyle( KBibTeX::kbibtexsettings.exportbibliographystyle );
                exporter = psExporter;
            }
            break;
        default:
            qDebug( "No supported export format selected" );
        }

        if ( exporter != NULL )
        {
            startProgress( label, exporter );
            result = exporter->save( iodevice, m_bibtexfile, errorLog );
            endProgress( exporter );

            if ( transform != NULL )
                delete transform;
            delete exporter;
        }

        setEnabled( TRUE );
        return result;
    }

    bool DocumentWidget::newElement( const QString& elementType )
    {
        if ( elementType.lower() == "string" )
        {
            QString name = QString( i18n( "NewString_%1" ) ).arg( m_newElementCounter++ );
            BibTeX::BibTeXString *string = new BibTeX::BibTeXString( name );
            if ( KBibTeX::StringWidget::execute( string, m_isReadOnly ) == QDialog::Accepted )
            {
                new KBibTeX::DocumentListViewItem( string, m_listViewElements );
                m_bibtexfile->appendElement( string );
                return TRUE;
            }
            else
            {
                delete string;
                return FALSE;
            }
        }
        else if ( elementType.lower() == "comment" )
        {
            BibTeX::BibTeXComment * comment = new BibTeX::BibTeXComment( i18n( "Put your comment here..." ) );
            if ( KBibTeX::CommentWidget::execute( comment, m_isReadOnly ) == QDialog::Accepted )
            {
                new KBibTeX::DocumentListViewItem( comment, m_listViewElements );
                m_bibtexfile->appendElement( comment );
                return TRUE;
            }
            else
            {
                delete comment;
                return FALSE;
            }
        }
        else
        {
            QString name = QString( i18n( "NewEntry_%1" ) ).arg( m_newElementCounter++ );
            BibTeX::BibTeXEntry *entry = new BibTeX::BibTeXEntry( elementType, name );
            if ( KBibTeX::EntryWidget::execute( entry, m_bibtexfile, m_isReadOnly ) == QDialog::Accepted )
            {
                new KBibTeX::DocumentListViewItem( entry, m_listViewElements );
                m_bibtexfile->appendElement( entry );
                return TRUE;
            }
            else
            {
                delete entry;
                return FALSE;
            }
        }
    }

    void DocumentWidget::updateViews()
    {
        m_listViewElements->updateVisiblity();
    }

    void DocumentWidget::setAddElementMenu( KPopupMenu *menu )
    {
        m_searchBar->setAddElementMenu( menu );
    }

    void DocumentWidget::setListViewContextMenu( KPopupMenu *menu )
    {
        m_listViewElements->setListViewContextMenu( menu );
    }

    void DocumentWidget::deferredInitialization()
    {
        m_listViewElements->deferredInitialization();
    }

    void DocumentWidget::editElement()
    {
        m_listViewElements->editElement( );
    }

    void DocumentWidget::deleteElements()
    {
        if ( !m_isReadOnly && currentPage() != m_sourceView )
        {
            m_listViewElements->deleteSelected();
            emit modified( );
        }
    }
    void DocumentWidget::cutElements()
    {
        if ( !m_isReadOnly && currentPage() != m_sourceView )
        {
            m_listViewElements->copySelected();
            m_listViewElements->deleteSelected();
            emit modified( );
        }
    }

    void DocumentWidget::copyElements()
    {
        if ( currentPage() != m_sourceView )
        {
            m_listViewElements->copySelected();
        }
    }

    void DocumentWidget::copyRefElements()
    {
        if ( currentPage() != m_sourceView )
        {
            m_listViewElements->copyRefSelected();
        }
    }

    void DocumentWidget::pasteElements()
    {
        if ( !m_isReadOnly && currentPage() != m_sourceView && m_listViewElements->paste() );
        emit modified( );
    }

    void DocumentWidget::setReadOnly( bool isReadOnly )
    {
        m_isReadOnly = isReadOnly;
        m_listViewElements->setReadOnly( m_isReadOnly );
        m_sourceView->setReadOnly( m_isReadOnly );
    }

    void DocumentWidget::slotShowProgress( int current, int total )
    {
        if ( m_progressDialog != NULL )
        {
            m_progressDialog->setProgress( current, total );
            qApp->processEvents();
        }
    }

    void DocumentWidget::startProgress( const QString & label, QObject * progressFrom )
    {
        m_progressDialog = new QProgressDialog( this );
        m_progressDialog->setLabelText( label );
        connect( progressFrom, SIGNAL( progress( int, int ) ), this, SLOT( slotShowProgress( int, int ) ) );
        connect( m_progressDialog, SIGNAL( canceled() ), progressFrom, SLOT( cancel( ) ) );
        QApplication::setOverrideCursor( Qt::waitCursor );
    }

    void DocumentWidget::endProgress( QObject * progressFrom )
    {
        disconnect( progressFrom, SIGNAL( progress( int, int ) ), this, SLOT( slotShowProgress( int, int ) ) );
        disconnect( m_progressDialog, SIGNAL( canceled() ), progressFrom, SLOT( cancel( ) ) );
        delete m_progressDialog;
        m_progressDialog = NULL;
        QApplication::restoreOverrideCursor();
    }

    void DocumentWidget::searchInternet( const QString& searchURL )
    {
        DocumentListViewItem * item = dynamic_cast<KBibTeX::DocumentListViewItem*>( m_listViewElements->selectedItem() );
        if ( item == NULL )
            item = dynamic_cast<KBibTeX::DocumentListViewItem*>( m_listViewElements->currentItem() );

        if ( item != NULL )
            searchInternet( item->element(), searchURL );
    }

    void DocumentWidget::searchInternet( BibTeX::BibTeXElement * element, const QString& searchURL )
    {
        QString queryString = QString::null;

        BibTeX::BibTeXEntry* entry = dynamic_cast<BibTeX::BibTeXEntry*>( element );
        if ( entry != NULL )
        {
            BibTeX::BibTeXEntryField * field = entry->getField( BibTeX::BibTeXEntryField::ftTitle );
            if ( field && !field->value() ->isEmpty() )
                queryString = field->value() ->plainString();
        }
        else
        {
            BibTeX::BibTeXComment * comment = dynamic_cast<BibTeX::BibTeXComment*>( element );
            if ( comment != NULL )
                queryString = comment->text();
            else
            {
                BibTeX::BibTeXString * string = dynamic_cast<BibTeX::BibTeXString*>( element );
                if ( string != NULL && !string->value() ->isEmpty() )
                    queryString = string->value() ->plainString();
                else
                    qDebug( "Not yet supported" );
            }
        }

        if ( queryString != QString::null )
            kapp->invokeBrowser( QString( searchURL ).arg( queryString.replace( '{', "" ).replace( '}', "" ) ) );
    }

    void DocumentWidget::updateFromGUI()
    {
        BibTeX::BibTeXFile * file = NULL;
        if ( currentPage() == m_sourceView )
            file = m_sourceView->getBibTeXFile();
        else if ( currentPage() == m_verticalContainer )
            file = m_listViewElements->getBibTeXFile();

        if ( file != NULL && file != m_bibtexfile )
        {
            delete m_bibtexfile;
            m_bibtexfile = file;
        }
    }

    void DocumentWidget::slotSelectionChanged()
    {
        int numSelected = 0;
        QListViewItemIterator it( m_listViewElements, QListViewItemIterator::Selected );
        while ( it.current() && numSelected < 3 )
        {
            numSelected++;
            it++;
        }

        emit listViewSelectionChanged( numSelected );
    }

    void DocumentWidget::slotPreviewElement( QListViewItem * item )
    {
        if ( item == NULL )
            item = m_listViewElements->selectedItem();
        if ( item == NULL )
            item = m_listViewElements->currentItem();
        if ( item != NULL )
        {
            DocumentListViewItem * dlvi = dynamic_cast<KBibTeX::DocumentListViewItem*>( item );
            if ( dlvi != NULL && m_listViewElements->isEnabled() )
            {
                BibTeX::BibTeXElement * element = dlvi->element();
                BibTeX::BibTeXXSLTransform *transform = new BibTeX::BibTeXXSLTransform( kbibtexsettings.xsltStylesheetHTML );
                if ( transform != NULL )
                {
                    BibTeX::BibTeXFileExporterXSLT * exporter = new BibTeX::BibTeXFileExporterXSLT( transform );
                    QBuffer buffer;
                    buffer.open( IO_WriteOnly );
                    bool result = exporter->save( &buffer, element );
                    buffer.close();

                    if ( result )
                    {
                        buffer.open( IO_ReadOnly );
                        QTextStream htmlTS( &buffer );
                        htmlTS.setEncoding( QTextStream::UnicodeUTF8 );
                        QString htmlText = htmlTS.read();
                        buffer.close();
                        QString text = htmlText.remove( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" ).append( "</qt>" ).prepend( "<qt>" ).replace( '{', "" ).replace( '}', "" );
                        m_labelPreview->setText( text );
                    }

                    delete exporter;
                    delete transform;
                }
            }
        }
    }

    void DocumentWidget::slotTabChanged( QWidget *tab )
    {
        setEnabled( FALSE );

        if ( tab == m_sourceView )
        {
            // switching from list view to source view
            BibTeX::BibTeXFile * bibtexFile = m_listViewElements->getBibTeXFile();
            if ( bibtexFile != NULL )
            {
                // check whether the selected widget has changed the bibtex file
                if ( bibtexFile != m_bibtexfile )
                {
                    delete m_bibtexfile;
                    m_bibtexfile = bibtexFile;
                }
                m_sourceView->setBibTeXFile( m_bibtexfile );
            }
        }
        else if ( tab == m_verticalContainer )
        {
            // switching from source view to list view
            BibTeX::BibTeXFile * bibtexFile = m_sourceView->getBibTeXFile();
            if ( bibtexFile != NULL )
            {
                // check whether the selected widget has changed the bibtex file
                if ( bibtexFile != m_bibtexfile )
                {
                    delete m_bibtexfile;
                    m_bibtexfile = bibtexFile;
                }
                m_listViewElements->setBibTeXFile( m_bibtexfile );
            }
            m_listViewElements->setFocus();
        }

        setEnabled( TRUE );

        if ( tab == m_sourceView )
            m_sourceView->setFocus();
        else if ( tab == m_verticalContainer )
            m_listViewElements->setFocus();
    }

    void DocumentWidget::slotModified()
    {
        if ( isEnabled() )
            emit modified( );
    }

}
#include "documentwidget.moc"
