/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qtextedit.h>

#include <kdialog.h>
#include <klocale.h>
#include <kiconloader.h>

#include <bibtexelement.h>
#include <bibtexentry.h>
#include <bibtexvalue.h>
#include <valuewidget.h>
#include "fieldlineedit.h"

namespace KBibTeX
{

    FieldLineEdit::FieldLineEdit( const QString& caption, InputType inputType, bool isReadOnly, QWidget *parent, const char *name )
            : QWidget( parent, name ), m_value( new BibTeX::BibTeXValue() ), m_caption( caption ), m_lineEdit( NULL ), m_textEdit( NULL ), m_isReadOnly( isReadOnly ), m_enabled( TRUE ), m_inputType( inputType ), m_isModified( FALSE )
    {
        setupGUI();
    }


    FieldLineEdit::~FieldLineEdit()
    {
        delete m_value;
    }

    void FieldLineEdit::setValue( const BibTeX::BibTeXValue *value )
    {
        delete m_value;

        if ( value != NULL )
            m_value = new BibTeX::BibTeXValue( value );
        else
            m_value = new BibTeX::BibTeXValue();

        updateGUI();
        m_isModified = FALSE;
    }

    BibTeX::BibTeXValue *FieldLineEdit::value()
    {
        if ( m_value->isEmpty() )
            return NULL;
        else
            return new BibTeX::BibTeXValue( m_value );
    }

    void FieldLineEdit::setEnabled( bool enabled )
    {
        m_enabled = enabled;
        updateGUI();
    }

    QString FieldLineEdit::caption()
    {
        return m_caption;
    }

    bool FieldLineEdit::isEmpty()
    {
        return m_value->isEmpty();
    }

    bool FieldLineEdit::isModified()
    {
        return m_isModified;
    }

    void FieldLineEdit::installEventFilters( QWidget *listener )
    {
        switch ( m_inputType )
        {
        case itSingleLine: m_lineEdit->installEventFilter( listener ); break;
        case itMultiLine: m_textEdit->installEventFilter( listener ); break;
        }
        m_pushButtonComplex->installEventFilter( listener );
        m_pushButtonString->installEventFilter( listener );
    }

    FieldLineEdit::ErrorType FieldLineEdit::error()
    {
        ErrorType result = etNoError;
        for ( BibTeX::BibTeXValue::ValueItems::const_iterator it = m_value->begin();result == etNoError && it != m_value->end(); it++ )
        {
            if ( ( *it ) ->isStringKey() && BibTeX::BibTeXValue::checkIsStringKey( ( *it ) ->text() ) == FALSE )
                result = etInvalidStringKey;
        }

        return result;
    }

    void FieldLineEdit::slotTextChanged( )
    {
        QString text;
        switch ( m_inputType )
        {
        case itSingleLine: text = m_lineEdit->text(); break;
        case itMultiLine: text = m_textEdit->text(); break;
        }

        if ( m_value->count() <= 1 )
        {
            m_value->clear();
            if ( !text.isEmpty() )
            {
                BibTeX::BibTeXValueItem * item = new BibTeX::BibTeXValueItem( text, m_pushButtonString->isOn() );
                m_value->add( item );
                m_isModified = TRUE;
            }
        }

        updateGUI();
    }

    void FieldLineEdit::slotStringToggled()
    {
        if ( m_value->count() == 1 )
            m_value->first() ->setIsStringKey( m_pushButtonString->isOn() );

        updateGUI();
        emit textChanged();
        m_isModified = TRUE;
    }

    void FieldLineEdit::slotComplexClicked()
    {
        if ( ValueWidget::execute( m_caption, m_value, m_isReadOnly, this ) == QDialog::Accepted )
        {
            updateGUI();
            emit textChanged();
            m_isModified = TRUE;
        }
    }

    void FieldLineEdit::setupGUI()
    {
        m_pushButtonString = new QPushButton( this );
        m_pushButtonString->setIconSet( QIconSet( SmallIcon( "flag" ) ) );
        m_pushButtonString->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
        m_pushButtonString->setToggleButton( TRUE );
        QToolTip::add( m_pushButtonString, QString( i18n( "Set \"%1\" to be a string key" ) ).arg( m_caption ) );
        connect( m_pushButtonString, SIGNAL( clicked() ), this, SLOT( slotStringToggled() ) );

        m_pushButtonComplex = new QPushButton( this );
        m_pushButtonComplex->setIconSet( QIconSet( SmallIcon( "leftjust" ) ) );
        m_pushButtonComplex->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
        QToolTip::add( m_pushButtonComplex, QString( i18n( "Edit \"%1\" as a concatenated value" ) ).arg( m_caption ) );
        connect( m_pushButtonComplex, SIGNAL( clicked() ), this, SLOT( slotComplexClicked() ) );

        QGridLayout * layout = NULL;
        switch ( m_inputType )
        {
        case itSingleLine:
            {
                layout = new QGridLayout( this, 2, 3, 0, KDialog::spacingHint() );
                m_lineEdit = new QLineEdit( this );
                m_lineEdit->setReadOnly( m_isReadOnly );
                layout->addWidget( m_lineEdit, 0, 0 );
                setFocusProxy( m_lineEdit );
                layout->addWidget( m_pushButtonString, 0, 1 );
                layout->addWidget( m_pushButtonComplex, 0, 2 );
                QWidget::setTabOrder( m_lineEdit, m_pushButtonString );
            }
            break;
        case itMultiLine:
            {
                layout = new QGridLayout( this, 3, 2, 0, KDialog::spacingHint() );
                layout->setRowStretch( 2, 1 );
                m_textEdit = new QTextEdit( this );
                m_textEdit->setReadOnly( m_isReadOnly );
                layout->addMultiCellWidget( m_textEdit, 0, 2, 0, 0 );
                setFocusProxy( m_textEdit );
                layout->addWidget( m_pushButtonString, 0, 1 );
                layout->addWidget( m_pushButtonComplex, 1, 1 );
                QWidget::setTabOrder( m_textEdit, m_pushButtonString );
            }
            break;
        }

        QWidget::setTabOrder( m_pushButtonString, m_pushButtonComplex );
        layout->setRowStretch( layout->numRows() - 1, 1 );

        enableSignals( TRUE );
    }

    void FieldLineEdit::updateGUI()
    {
        enableSignals( FALSE );

        bool inputEnable = FALSE;
        switch ( m_inputType )
        {
        case itSingleLine: inputEnable = m_lineEdit->isEnabled(); break;
        case itMultiLine: inputEnable = m_textEdit->isEnabled();break;
        }

        inputEnable = ( m_value->count() <= 1 ) && m_enabled;
        m_pushButtonString->setEnabled( inputEnable && !m_isReadOnly && m_enabled );

        if ( m_value->count() == 1 )
        {
            BibTeX::BibTeXValueItem * item = m_value->first();
            m_pushButtonString->setOn( item->isStringKey() );
            switch ( m_inputType )
            {
            case itSingleLine:
                if ( QString::compare( m_lineEdit->text(), item->text() ) != 0 )
                    m_lineEdit->setText( item->text() );
                break;
            case itMultiLine:
                if ( QString::compare( m_textEdit->text(), item->text() ) != 0 )
                    m_textEdit->setText( item->text() );
                break;
            }
        }
        else
        {
            switch ( m_inputType )
            {
            case itSingleLine: m_lineEdit->setText( "" ); break;
            case itMultiLine: m_textEdit->setText( "" ); break;
            }
        }

        m_pushButtonComplex->setEnabled( m_enabled );

        switch ( m_inputType )
        {
        case itSingleLine:
            m_lineEdit->setEnabled( inputEnable );
            m_lineEdit->setReadOnly( m_isReadOnly );
            break;
        case itMultiLine:
            m_textEdit->setEnabled( inputEnable );
            m_textEdit->setReadOnly( m_isReadOnly );
            break;
        }

        enableSignals( TRUE );
    }

    void FieldLineEdit::enableSignals( bool enabled )
    {
        switch ( m_inputType )
        {
        case itSingleLine:
            if ( enabled )
            {
                connect( m_lineEdit, SIGNAL( textChanged( const QString& ) ), this, SLOT( slotTextChanged( ) ) );
                connect( m_lineEdit, SIGNAL( textChanged( const QString& ) ), this, SIGNAL( textChanged( ) ) );
            }
            else
            {
                disconnect( m_lineEdit, SIGNAL( textChanged( const QString& ) ), this, SLOT( slotTextChanged( ) ) );
                disconnect( m_lineEdit, SIGNAL( textChanged( const QString& ) ), this, SIGNAL( textChanged( ) ) );
            }
            break;
        case itMultiLine:
            if ( enabled )
            {
                connect( m_textEdit, SIGNAL( textChanged( ) ), this, SLOT( slotTextChanged( ) ) );
                connect( m_textEdit, SIGNAL( textChanged( ) ), this, SIGNAL( textChanged( ) ) );
            }
            else
            {
                disconnect( m_textEdit, SIGNAL( textChanged( ) ), this, SLOT( slotTextChanged( ) ) );
                disconnect( m_textEdit, SIGNAL( textChanged( ) ), this, SIGNAL( textChanged( ) ) );
            }
            break;
        }
    }

}
#include "fieldlineedit.moc"
