/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qstring.h>
#include <qregexp.h>
#include <qlayout.h>
#include <qheader.h>
#include <qlistview.h>
#include <qtooltip.h>
#include <qpushbutton.h>

#include <kdialog.h>
#include <klocale.h>
#include <kiconloader.h>

#include <bibtexvalue.h>
#include <valuewidget.h>
#include "fieldlistview.h"

namespace KBibTeX
{

    FieldListView::FieldListView( const QString& caption, bool isReadOnly, QWidget *parent, const char *name )
            : QWidget( parent, name ), m_value( new BibTeX::BibTeXValue() ), m_caption( caption ), m_isReadOnly( isReadOnly ), m_enabled( TRUE ), m_isModified( FALSE ), m_newValueCounter( 0 )
    {
        setupGUI();
    }


    FieldListView::~FieldListView()
    {
        delete m_value;
    }

    void FieldListView::setValue( const BibTeX::BibTeXValue *value )
    {
        if ( value != m_value )
        {
            if ( m_value != NULL )
                delete m_value;

            if ( value != NULL )
                m_value = new BibTeX::BibTeXValue( value );
            else
                m_value = new BibTeX::BibTeXValue();

            updateListView();
            updateGUI();

            m_isModified = FALSE;
        }
    }

    BibTeX::BibTeXValue *FieldListView::value()
    {
        if ( m_value->isEmpty() )
            return NULL;
        else
            return new BibTeX::BibTeXValue( m_value );
    }

    void FieldListView::setEnabled( bool enabled )
    {
        m_enabled = enabled;
        updateGUI();
    }

    QString FieldListView::caption()
    {
        return m_caption;
    }


    bool FieldListView::isEmpty()
    {
        return m_value != NULL ? m_value->isEmpty() : TRUE;
    }

    bool FieldListView::isModified()
    {
        return m_isModified;
    }

    void FieldListView::installEventFilters( QWidget *listener )
    {
        m_listViewElements->installEventFilter( listener );
    }

    void FieldListView::updateGUI()
    {
        if ( m_value != NULL && isSimple() )
        {
            bool isElementSelected = m_listViewElements->selectedItem() != NULL;
            m_pushButtonAdd->setEnabled( !m_isReadOnly );
            m_pushButtonEdit->setEnabled( !m_isReadOnly && isElementSelected );
            m_pushButtonDelete->setEnabled( !m_isReadOnly && isElementSelected );
            m_pushButtonUp->setEnabled( !m_isReadOnly && isElementSelected && m_listViewElements->selectedItem() != m_listViewElements->firstChild() );
            m_pushButtonDown->setEnabled( !m_isReadOnly && isElementSelected && m_listViewElements->selectedItem() != m_listViewElements->lastItem() );
            m_listViewElements->setEnabled( !m_isReadOnly );
        }
        else
        {
            m_pushButtonAdd->setEnabled( FALSE );
            m_pushButtonEdit->setEnabled( FALSE );
            m_pushButtonDelete->setEnabled( FALSE );
            m_pushButtonUp->setEnabled( FALSE );
            m_pushButtonDown->setEnabled( FALSE );
            m_listViewElements->setEnabled( FALSE );
        }
    }

    void FieldListView::slotAdd()
    {
        if ( isSimple() )
        {
            QListViewItem * item = new QListViewItem( m_listViewElements, m_listViewElements->lastItem(), QString( i18n( "New%1%2" ) ).arg( m_caption ).arg( m_newValueCounter++ ) );
            item->setRenameEnabled( 0, TRUE );
            m_listViewElements->setSelected( item, TRUE );
            updateValue();
            slotEdit();
            m_isModified = TRUE;
        }
    }

    void FieldListView::slotEdit()
    {
        if ( isSimple() )
        {
            QListViewItem * item = m_listViewElements->selectedItem();
            if ( item != NULL )
                item->startRename( 0 );
            else
            {
                item = m_listViewElements->currentItem();
                if ( item != NULL )
                    item->startRename( 0 );
            }
        }
    }

    void FieldListView::slotDelete()
    {
        QListViewItem * item = m_listViewElements->selectedItem();
        if ( isSimple() && item != NULL )
        {
            delete item;
            updateValue();
            updateGUI();

            m_isModified = TRUE;
        }
    }

    void FieldListView::slotUp()
    {
        QListViewItem * item = m_listViewElements->selectedItem();
        if ( isSimple() && !m_listViewElements->isRenaming() && item != NULL && item -> itemAbove() != NULL )
        {
            item->itemAbove() ->moveItem( item );
            updateValue();
            updateGUI();

            m_isModified = TRUE;
        }
    }

    void FieldListView::slotDown()
    {
        QListViewItem * item = m_listViewElements->selectedItem();
        if ( isSimple() && !m_listViewElements->isRenaming() && item != NULL && item -> itemBelow() != NULL )
        {
            item->moveItem( item->itemBelow() );
            updateValue();
            updateGUI();

            m_isModified = TRUE;
        }
    }

    void FieldListView::slotComplex()
    {
        if ( !m_listViewElements->isRenaming() && ValueWidget::execute( m_caption, m_value, m_isReadOnly, this ) == QDialog::Accepted )
        {
            updateListView();
            updateGUI();

            m_isModified = TRUE;
        }
    }

    void FieldListView::slotListViewDoubleClicked( QListViewItem *lvi )
    {
        if ( lvi == NULL )
            slotAdd();
        else
            slotEdit();
    }

    void FieldListView::slotItemRenamed( QListViewItem * item, int /*col*/, const QString & text )
    {
        if ( text.isEmpty() && isSimple() && item != NULL )
        {
            delete item;
            updateGUI();
        }

        updateValue();
        m_isModified = TRUE;
    }

    void FieldListView::setupGUI()
    {
        QGridLayout * layout = new QGridLayout( this, 7, 2, 0, KDialog::marginHint() );
        layout->setRowStretch( 6, 1 );

        m_listViewElements = new QListView( this );
        layout->addMultiCellWidget( m_listViewElements, 0, 6, 0, 0 );
        m_listViewElements->setDefaultRenameAction( QListView::Accept );
        m_listViewElements->addColumn( m_caption );
        m_listViewElements->setSorting( -1, TRUE );
        m_listViewElements->header() ->setClickEnabled( FALSE );
        m_listViewElements->header() ->setStretchEnabled( TRUE, 0 );
        connect( m_listViewElements, SIGNAL( selectionChanged() ), this, SLOT( updateGUI() ) );
        connect( m_listViewElements, SIGNAL( clicked( QListViewItem * ) ), this, SLOT( updateGUI() ) );
        connect( m_listViewElements, SIGNAL( doubleClicked( QListViewItem *, const QPoint &, int ) ), this, SLOT( slotListViewDoubleClicked( QListViewItem * ) ) );
        connect( m_listViewElements, SIGNAL( currentChanged( QListViewItem * ) ), this, SLOT( updateGUI() ) );
        connect( m_listViewElements, SIGNAL( itemRenamed( QListViewItem*, int, const QString& ) ), this, SLOT( slotItemRenamed( QListViewItem*, int, const QString& ) ) );

        m_pushButtonAdd = new QPushButton( i18n( "Add" ), this );
        layout->addWidget( m_pushButtonAdd, 0, 1 );
        m_pushButtonAdd->setIconSet( QIconSet( SmallIcon( "add" ) ) );
        connect( m_pushButtonAdd, SIGNAL( clicked() ), this, SLOT( slotAdd() ) );
        QToolTip::add( m_pushButtonAdd, QString( i18n( "Add new ”%1“ item" ) ).arg( m_caption ) );

        m_pushButtonEdit = new QPushButton( i18n( "Edit" ), this );
        layout->addWidget( m_pushButtonEdit, 1, 1 );
        m_pushButtonEdit->setIconSet( QIconSet( SmallIcon( "edit" ) ) );
        connect( m_pushButtonEdit, SIGNAL( clicked() ), this, SLOT( slotEdit() ) );
        QToolTip::add( m_pushButtonEdit, QString( i18n( "Edit current ”%1“ item" ) ).arg( m_caption ) );

        m_pushButtonDelete = new QPushButton( i18n( "Delete" ), this );
        layout->addWidget( m_pushButtonDelete, 2, 1 );
        m_pushButtonDelete->setIconSet( QIconSet( SmallIcon( "editdelete" ) ) );
        connect( m_pushButtonDelete, SIGNAL( clicked() ), this, SLOT( slotDelete() ) );
        QToolTip::add( m_pushButtonDelete, QString( i18n( "Delete current ”%1“ item" ) ).arg( m_caption ) );

        m_pushButtonUp = new QPushButton( i18n( "Up" ), this );
        layout->addWidget( m_pushButtonUp, 3, 1 );
        m_pushButtonUp->setIconSet( QIconSet( SmallIcon( "up" ) ) );
        connect( m_pushButtonUp, SIGNAL( clicked() ), this, SLOT( slotUp() ) );
        QToolTip::add( m_pushButtonUp, QString( i18n( "Move current ”%1“ item up" ) ).arg( m_caption ) );

        m_pushButtonDown = new QPushButton( i18n( "Down" ), this );
        layout->addWidget( m_pushButtonDown, 4, 1 );
        m_pushButtonDown->setIconSet( QIconSet( SmallIcon( "down" ) ) );
        connect( m_pushButtonDown, SIGNAL( clicked() ), this, SLOT( slotDown() ) );
        QToolTip::add( m_pushButtonDown, QString( i18n( "Move current ”%1“ item down" ) ).arg( m_caption ) );

        m_pushButtonComplexEdit = new QPushButton( i18n( "Complex..." ), this );
        layout->addWidget( m_pushButtonComplexEdit, 5, 1 );
        m_pushButtonComplexEdit->setIconSet( QIconSet( SmallIcon( "leftjust" ) ) );
        connect( m_pushButtonComplexEdit, SIGNAL( clicked() ), this, SLOT( slotComplex() ) );
        QToolTip::add( m_pushButtonComplexEdit, QString( i18n( "Edit current ”%1“ item as a concatenated string" ) ).arg( m_caption ) );
    }

    void FieldListView::updateValue()
    {
        QString text;

        for ( QListViewItemIterator it( m_listViewElements ); it.current(); it++ )
        {
            if ( !text.isEmpty() )
                text.append( " and " );
            text.append( it.current() ->text( 0 ) );
        }

        m_value->clear();
        if ( !text.isEmpty() )
        {
            BibTeX::BibTeXValueItem * item = new BibTeX::BibTeXValueItem ( text, FALSE );
            m_value->add( item );
        }
    }

    void FieldListView::updateListView()
    {
        m_listViewElements->clear();
        if ( m_value->count() == 1 && !m_value->first() ->isStringKey() )
        {
            QStringList elements = QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", FALSE ), m_value->first() ->text() );
            for ( QStringList::Iterator it = elements.begin(); it != elements.end(); it++ )
            {
                QListViewItem *item = new QListViewItem( m_listViewElements, m_listViewElements->lastItem(), *it );
                item->setRenameEnabled( 0, TRUE );
            }
        }

    }

    bool FieldListView::isSimple()
    {
        return m_value->count() == 0 || ( m_value->count() == 1 && !m_value->first() ->isStringKey() );
    }

}
#include "fieldlistview.moc"
