/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <string>

#include <qstring.h>
#include <qsplitter.h>
#include <qtimer.h>
#include <qptrlist.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qbuffer.h>
#include <qtoolbutton.h>
#include <qtabwidget.h>
#include <qlabel.h>
#include <qtextedit.h>
#include <qclipboard.h>
#include <kinstance.h>
#include <kaction.h>
#include <kactionclasses.h>
#include <kstdaction.h>
#include <klistview.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <ktempfile.h>
#include <kconfigdialog.h>
#include <kglobalsettings.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <kiconloader.h>
#include <kapplication.h>
#include <kxmlguifactory.h>
#include <kpopupmenu.h>
#include <kio/netaccess.h>

#include <bibtexcomment.h>
#include <bibtexxsltransform.h>
#include <bibtexstring.h>
#include <bibtexentry.h>
#include <bibtexelement.h>
#include <documentlistviewitem.h>
#include <kbibtexsettingsdlg.h>
#include <documentlistview.h>
#include <bibtexfileexporter.h>
#include <bibtexfileexporterbibtex.h>
#include <bibtexfileexporterxml.h>
#include <bibtexfileexporterxslt.h>
#include <bibtexfileimporter.h>
#include <bibtexfileimporterbibtex.h>
#include <settings.h>
#include <encoderlatex.h>
#include <documentwidget.h>
#include <settings.h>

#include "kbibtex_part.h"

KBibTeX::Settings KBibTeX::kbibtexsettings;

KBibTeXPart::KBibTeXPart( QWidget *parentWidget, const char *widgetName,
                          QObject *parent, const char *name )
        : KParts::ReadWritePart( parent, name ), m_defInitCounter( 0 ), m_settingsDlg( NULL ), m_bibTeXXSLTHTML( NULL )
{
    // we need an instance
    setInstance( KBibTeXPartFactory::instance() );

    // set our XML-UI resource file
    setXMLFile( "kbibtex_part.rc" );

    // setup GUI elements
    setupGUI( parentWidget, widgetName );

    // create our actions
    setupActions();

    // we are read-write by default
    setReadWrite( TRUE );

    // we are not modified since we haven't done anything yet
    setModified( FALSE );

    // read configuration
    readSettings();

    QTimer::singleShot( 100, this, SLOT( slotDeferredInitialization() ) );
}

KBibTeXPart::~KBibTeXPart()
{
    // save configuration
    writeSettings();

    if ( m_settingsDlg != NULL )
        delete m_settingsDlg;

    if ( m_bibTeXXSLTHTML != NULL )
        delete m_bibTeXXSLTHTML;

    BibTeX::EncoderLaTeX::deleteCurrentEncoderLaTeX();
}

void KBibTeXPart::setReadWrite( bool rw )
{
    if ( rw )
    {
        connect( m_documentWidget, SIGNAL( modified( ) ), this, SLOT( setModified( ) ) );
        m_actionEditElement->setText( "&Edit" );
    }
    else
    {
        disconnect( m_documentWidget, SIGNAL( modified( ) ), this, SLOT( setModified( ) ) );
        m_actionEditElement->setText( "&View" );
    }
    ReadWritePart::setReadWrite( rw );
    m_documentWidget->setReadOnly( !rw );
}

void KBibTeXPart::setModified( bool modified )
{
    // get a handle on our Save action and make sure it is valid
    KAction * save = actionCollection() ->action( KStdAction::stdName( KStdAction::Save ) );

    if ( save )
    {
        // if so, we either enable or disable it based on the current
        // state
        save->setEnabled( modified );

        // in any event, we want our parent to do it's thing
        KParts::ReadWritePart::setModified( modified );
    }
}

bool KBibTeXPart::queryClose()
{
    if ( !isReadWrite() || !isModified() )
        return true;

    QString docName = url().fileName();
    if ( docName.isEmpty() ) docName = i18n( "Untitled" );

    int res = KMessageBox::warningYesNoCancel( widget(),
              i18n( "The document \"%1\" has been modified.\n"
                    "Do you want to save your changes or discard them?" ).arg( docName ),
              i18n( "Close Document" ), KStdGuiItem::save(), KStdGuiItem::discard() );

    bool abortClose = false;
    bool handled = false;

    switch ( res )
    {
    case KMessageBox::Yes :
        sigQueryClose( &handled, &abortClose );
        if ( !handled )
        {
            if ( m_url.isEmpty() )
            {
                return saveAs();
            }
            else
            {
                save();
            }
        }
        else if ( abortClose ) return false;
        return waitSaveComplete();
    case KMessageBox::No :
        return true;
    default:
        return false;
    }

}

bool KBibTeXPart::closeURL()
{
    if ( isReadWrite() && isModified() )
    {
        if ( !queryClose() )
            return false;
    }
    // Not modified => ok and delete temp file.
    return KParts::ReadOnlyPart::closeURL();
}

bool KBibTeXPart::openFile()
{
    return m_documentWidget->open( m_file, FALSE );
}

/**
 * Save document to a filename as given in String m_file.
 */
bool KBibTeXPart::saveFile()
{
    if ( isReadWrite() == FALSE )
        return FALSE;

    if ( !url().isValid() || url().isEmpty() )
        return saveAs();
    else
        return m_documentWidget->save( m_file );
}

void KBibTeXPart::slotFileSaveAs()
{
    saveAs();
}

void KBibTeXPart::slotFileOpen()
{
    KURL url = KFileDialog::getOpenURL( QDir::currentDirPath(), "*.bib|" + i18n( "BibTeX (*.bib)" ), widget() );

    if ( KIO::NetAccess::exists( url, TRUE, widget() ) && openURL( url ) )
        m_recent->addURL( url );
}

bool KBibTeXPart::saveAs()
{
    bool result = FALSE;

    KURL saveURL = KFileDialog::getSaveURL( url().isLocalFile() ? url().url() : url().fileName(), "*.bib|" + i18n( "BibTeX (*.bib)" ), widget() );

    if ( saveURL.isValid() && !saveURL.isEmpty() )
    {

        if ( KIO::NetAccess::exists( saveURL, FALSE, widget() ) && KMessageBox::warningContinueCancel( widget(), i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" ).arg( saveURL.filename() ), QString::null, i18n( "Overwrite" ) ) != KMessageBox::Continue )
            return result;

        if ( ReadWritePart::saveAs( saveURL ) )
        {
            m_recent->addURL( url() );
            result = TRUE;
        }
    }

    return result;
}

void KBibTeXPart::slotFileExport()
{
    KURL exportURL = KFileDialog::getSaveURL( QString::null, "*.pdf|" + i18n( "Portable Document File (*.pdf)" ) + "\n*.ps|" + i18n( "PostScript (*.ps)" ) + "\n*.xml|" + i18n( "eXtended Markup Language (*.xml)" ) + "\n*.html|" + i18n( "Hypertext Markup Langage (*.html)" ), widget() );

    if ( exportURL.isValid() && !exportURL.isEmpty() )
    {
        if ( KIO::NetAccess::exists( exportURL, FALSE, widget() ) && KMessageBox::warningContinueCancel( widget(), i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" ).arg( exportURL.filename() ), QString::null, i18n( "Overwrite" ) ) != KMessageBox::Continue )
            return ;

        KTempFile tempBibFile = KTempFile( locateLocal( "tmp", "bibexport" ) );
        tempBibFile.setAutoDelete( TRUE );
        KBibTeX::Settings::FileFormat format = KBibTeX::Settings::formatBibTeX;
        if ( exportURL.fileName().endsWith( ".pdf" ) )
            format = KBibTeX::Settings::formatPDF;
        else if ( exportURL.fileName().endsWith( ".ps" ) )
            format = KBibTeX::Settings::formatPS;
        else if ( exportURL.fileName().endsWith( ".xml" ) )
            format = KBibTeX::Settings::formatXML;
        else if ( exportURL.fileName().endsWith( ".html" ) )
            format = KBibTeX::Settings::formatHTML;

        QStringList errorLog;
        bool error = !m_documentWidget->save( tempBibFile.file(), format, i18n( "Exporting" ), &errorLog ) ;
        if ( !error )
        {
            tempBibFile.close();
            error = !KIO::NetAccess::file_copy( KURL( tempBibFile.name() ), exportURL, -1, TRUE, FALSE, widget() );
        }

        if ( error )
        {
#if (KDE_VERSION_MINOR>=4) && (KDE_VERSION_MAJOR>=3)
            KMessageBox::errorList( widget(), QString( i18n( "File could not be exported in \"%1\"." ) ).arg( exportURL.prettyURL() ).append( i18n( "\n\nSee the following error log for details." ) ), errorLog, i18n( "Exporting failed" ) );
#else
            KMessageBox::error( widget(), QString( i18n( "File could not be exported in \"%1\"." ) ).arg( exportURL.prettyURL() ), i18n( "Exporting failed" ) );
#endif

        }

    }

}

void KBibTeXPart::setupGUI( QWidget *parentWidget, const char *name )
{
    m_documentWidget = new KBibTeX::DocumentWidget( !isReadWrite(), parentWidget, name );

    // notify the part that this is our internal widget
    setWidget( m_documentWidget );
}

void KBibTeXPart::setupActions()
{
    KStdAction::open( this, SLOT( slotFileOpen() ), actionCollection() );
    if ( isReadWrite() )
    {
        KStdAction::save( this, SLOT( save() ), actionCollection() );
        KStdAction::saveAs( this, SLOT( slotFileSaveAs() ), actionCollection() );
    }
    m_actionFileExport = new KAction( i18n( "Export..." ), SmallIconSet( "goto" ), CTRL + Key_E, this, SLOT( slotFileExport() ), actionCollection(), "file_export" );

    m_actionEditCut = KStdAction::cut( m_documentWidget, SLOT( cutElements() ), actionCollection() );
    m_actionEditCopy = KStdAction::copy( m_documentWidget, SLOT( copyElements() ), actionCollection() );
    m_actionEditCopyRef = new KAction( i18n( "Copy &Reference" ), CTRL + SHIFT + Key_C, m_documentWidget, SLOT( copyRefElements() ), actionCollection(), "edit_copyref" );
    m_actionEditPaste = KStdAction::paste( m_documentWidget, SLOT( pasteElements() ), actionCollection() );

    m_menuSearchInternet = new KActionMenu( i18n( "Search Internet" ), actionCollection(), "search_internet" );
    connect( m_menuSearchInternet->popupMenu(), SIGNAL( activated( int ) ), this, SLOT( slotSearchInternet( int ) ) );

    m_actionPreferences = KStdAction::preferences( this, SLOT( slotPreferences() ), actionCollection() );

    new KAction( i18n( "Article" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_article" );
    new KAction( i18n( "Book" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_book" );
    new KAction( i18n( "Booklet" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_booklet" );
    new KAction( i18n( "Electronic (IEEE)" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_electronic" );
    new KAction( i18n( "InBook" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_inbook" );
    new KAction( i18n( "InCollection" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_incollection" );
    new KAction( i18n( "InProceedings" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_inproceedings" );
    new KAction( i18n( "Manual" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_manual" );
    new KAction( i18n( "MastersThesis" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_mastersthesis" );
    new KAction( i18n( "Misc" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_misc" );
    new KAction( i18n( "PhDThesis" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_phdthesis" );
    new KAction( i18n( "Proceedings" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_proceedings" );
    new KAction( i18n( "TechReport" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_techreport" );
    new KAction( i18n( "Unpublished" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_unpublished" );
    new KAction( i18n( "Comment" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_comment" );
    new KAction( i18n( "String" ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_string" );

    m_actionEditElement = new KAction( i18n( "&Edit" ), 0, m_documentWidget, SLOT( editElement() ), actionCollection(), "element_edit" );
    m_actionDeleteElement = new KAction( i18n( "&Delete" ), 0, m_documentWidget, SLOT( deleteElements() ), actionCollection(), "element_delete" );

    m_recent = new KRecentFilesAction( i18n( "Open &Recent Files" ), 0,
                                       this, SLOT( openURL( const KURL & ) ),
                                       actionCollection(), "file_open_recent" );

    m_actionShowComments = new KToggleAction( i18n( "Show &Comments" ), 0, this, SLOT( slotToggleShowSpecialElements() ), actionCollection(), "view_showcomments" );
    m_actionShowStrings = new KToggleAction( i18n( "Show &Strings" ), 0, this, SLOT( slotToggleShowSpecialElements() ), actionCollection(), "view_showstrings" );

    connect( m_documentWidget, SIGNAL( listViewSelectionChanged( int ) ), this, SLOT( slotUpdateMenu( int ) ) );

    actionCollection() ->setHighlightingEnabled( true );

    KActionMenu *acmElementNew = new KActionMenu( i18n( "&New" ), QIconSet( SmallIcon( "filenew" ) ), actionCollection(), "element_new" );
    acmElementNew->setEnabled( FALSE );
}

bool KBibTeXPart::newElement()
{
    if ( isReadWrite() && sender() )
    {
        QString elementType;

        if ( strncmp( sender() ->name(), "element_new_entry_", 18 ) == 0 )
            elementType = QString( sender() ->name() + 18 );
        else if ( strcmp( sender() ->name(), "element_new_comment" ) == 0 )
            elementType = "comment";
        else if ( strcmp( sender() ->name(), "element_new_string" ) == 0 )
            elementType = "string";
        else
            return FALSE;

        bool result = m_documentWidget->newElement( elementType );
        if ( result )
            setModified( TRUE );
        return result;
    }

    return FALSE;
}

void KBibTeXPart::readSettings()
{
    KConfig * config = KBibTeXPartFactory::instance() ->config();

    config->setGroup( "Session" );
    m_recent->loadEntries( config );

    KBibTeX::kbibtexsettings.load( config );

    if ( m_bibTeXXSLTHTML != NULL )
        delete m_bibTeXXSLTHTML;
    m_bibTeXXSLTHTML = new BibTeX::BibTeXXSLTransform( KBibTeX::kbibtexsettings.xsltStylesheetHTML );
}

void KBibTeXPart::writeSettings()
{
    KConfig * config = KBibTeXPartFactory::instance() ->config();

    config->setGroup( "Session" );
    m_recent->saveEntries( config );

    KBibTeX::kbibtexsettings.save( config );
}

void KBibTeXPart::slotUpdateMenu( int numSelectedItems )
{
    m_actionEditElement->setEnabled( numSelectedItems == 1 );
    m_actionDeleteElement->setEnabled( isReadWrite() && numSelectedItems > 0 );
    m_actionEditCut->setEnabled( isReadWrite() && numSelectedItems > 0 );
    m_actionEditCopy->setEnabled( numSelectedItems > 0 );
    m_actionEditCopyRef->setEnabled( numSelectedItems > 0 );
    m_menuSearchInternet->setEnabled( numSelectedItems == 1 );

    KPopupMenu *popup = m_menuSearchInternet->popupMenu();
    popup->clear();
    int i = 0;
    for ( KBibTeX::Settings::SearchURL * searchURL = KBibTeX::kbibtexsettings.searchURLs.first(); searchURL; searchURL = KBibTeX::kbibtexsettings.searchURLs.next() )
        popup->insertItem( searchURL->description, ++i );
}

void KBibTeXPart::slotSearchInternet( int id )
{
    m_documentWidget->searchInternet( KBibTeX::kbibtexsettings.searchURLs.at( id - 1 ) ->url );
}

void KBibTeXPart::slotToggleShowSpecialElements()
{
    KBibTeX::kbibtexsettings.showComments = m_actionShowComments->isChecked();
    KBibTeX::kbibtexsettings.showStrings = m_actionShowStrings->isChecked();

    m_documentWidget->updateViews();
}

bool KBibTeXPart::save()
{
    if ( !url().isValid() || url().isEmpty() )
        return saveAs();
    else
        return KParts::ReadWritePart::save();
}

void KBibTeXPart::slotPreferences()
{
    if ( !m_settingsDlg )
        m_settingsDlg = new KBibTeXSettingsDlg( widget(), "settings_dlg" );

    m_settingsDlg->exec();


}

void KBibTeXPart::slotDeferredInitialization()
{
    if ( factory() != NULL )
    {
        m_actionShowComments->setChecked( KBibTeX::kbibtexsettings.showComments );
        m_actionShowStrings->setChecked( KBibTeX::kbibtexsettings.showStrings );

        m_documentWidget->setAddElementMenu( static_cast<KPopupMenu*>( factory() -> container( "popup_newelements", this ) ) );
        m_documentWidget->setListViewContextMenu( static_cast<KPopupMenu*>( factory() -> container( "popup_bibtexlist", this ) ) );

        slotUpdateMenu( 0 );

        m_documentWidget->deferredInitialization();
    }
    else
    {
        m_defInitCounter++;

        if ( m_defInitCounter > 50 )
        {
            KGuiItem retry = KGuiItem( i18n( "Try again" ), "reload" );
            if ( KMessageBox::warningContinueCancel( widget(), i18n( "The KBibTeX part has problems to initialize itself." ), i18n( "KBibTeX Part" ), retry ) == KMessageBox::Cancel )
                return ;
            else
                m_defInitCounter = 0;
        }
        QTimer::singleShot( 100, this, SLOT( slotDeferredInitialization() ) );
    }
}

// It's usually safe to leave the factory code alone.. with the
// notable exception of the KAboutData data
#include <kaboutdata.h>
#include <klocale.h>

KInstance* KBibTeXPartFactory::s_instance = 0L;
KAboutData* KBibTeXPartFactory::s_about = 0L;

KBibTeXPartFactory::KBibTeXPartFactory()
        : KParts::Factory()
{
    // nothing
}

KBibTeXPartFactory::~KBibTeXPartFactory()
{
    delete s_instance;
    delete s_about;

    s_instance = 0L;
}

KParts::Part* KBibTeXPartFactory::createPartObject( QWidget *parentWidget, const char *widgetName,
        QObject *parent, const char *name,
        const char *classname, const QStringList & /* args */ )
{
    // Create an instance of our Part
    KBibTeXPart * obj = new KBibTeXPart( parentWidget, widgetName, parent, name );

    // See if we are to be read-write or not
    if ( QCString( classname ) == "KParts::ReadOnlyPart" )
        obj->setReadWrite( false );

    return obj;
}

KInstance* KBibTeXPartFactory::instance()
{
    if ( !s_instance )
    {
        s_about = new KAboutData( "kbibtexpart", I18N_NOOP( "KBibTeXPart" ), "0.1.4" );
        s_about->addAuthor( "Thomas Fischer", 0, "fischer@unix-ag.uni-kl.de" );
        s_instance = new KInstance( s_about );
    }
    return s_instance;
}

extern "C"
{
    void* init_libkbibtexpart()
    {
        return new KBibTeXPartFactory;
    }
};

#include "kbibtex_part.moc"

